Module: Mpp::Hints

Defined in:
lib/mpp/errors.rb

Overview

Default hint messages for payment errors.

Constant Summary collapse

PAYMENT_REQUIRED =
"Use a supported wallet to pay for this resource using one of the supported " \
"payment methods returned in the WWW-Authenticate header. See https://mpp.dev/tools/wallet.md"
MALFORMED_CREDENTIAL =
"Use a supported wallet to construct valid credentials for one of the supported " \
"payment methods returned in the WWW-Authenticate header. See https://mpp.dev/tools/wallet.md"
METHOD_UNSUPPORTED =
PAYMENT_REQUIRED